@@ -49,7 +49,7 @@ class TransferProgress(object):
     def __init__(self, tp):
 
         self.total_objects = tp.total_objects
-        """Total number objects to download"""
+        """Total number of objects to download"""
 
         self.indexed_objects = tp.indexed_objects
         """Objects which have been indexed"""
@@ -127,20 +127,6 @@ class Remote(object):
         self._remote = ptr
         self._stored_exception = None
 
-        # Build the callback structure
-        callbacks = ffi.new('git_remote_callbacks *')
-        callbacks.version = 1
-        callbacks.sideband_progress = self._sideband_progress_cb
-        callbacks.transfer_progress = self._transfer_progress_cb
-        callbacks.update_tips = self._update_tips_cb
-        callbacks.credentials = self._credentials_cb
-        # We need to make sure that this handle stays alive
-        self._self_handle = ffi.new_handle(self)
-        callbacks.payload = self._self_handle
-
-        err = C.git_remote_set_callbacks(self._remote, callbacks)
-        check_error(err)
-
     def __del__(self):
         C.git_remote_free(self._remote)
 
@@ -191,6 +177,14 @@ class Remote(object):
         err = C.git_remote_set_pushurl(self._remote, to_bytes(value))
         check_error(err)
 
+    def delete(self):
+        """Remove this remote
+
+        All remote-tracking branches and configuration settings for the remote will be removed.
+        """
+        err = C.git_remote_delete(self._remote)
+        check_error(err)
+
     def save(self):
         """save()
 
@@ -205,17 +199,47 @@ class Remote(object):
         Perform a fetch against this remote.
         """
 
+        # Get the default callbacks first
+        defaultcallbacks = ffi.new('git_remote_callbacks *')
+        err = C.git_remote_init_callbacks(defaultcallbacks, 1)
+        check_error(err)
+
+        # Build custom callback structure
+        callbacks = ffi.new('git_remote_callbacks *')
+        callbacks.version = 1
+        callbacks.sideband_progress = self._sideband_progress_cb
+        callbacks.transfer_progress = self._transfer_progress_cb
+        callbacks.update_tips = self._update_tips_cb
+        callbacks.credentials = self._credentials_cb
+        # We need to make sure that this handle stays alive
+        self._self_handle = ffi.new_handle(self)
+        callbacks.payload = self._self_handle
+
+        err = C.git_remote_set_callbacks(self._remote, callbacks)
+        try:
+            check_error(err)
+        except:
+            self._self_handle = None
+            raise
+
         if signature:
             ptr = signature._pointer[:]
         else:
             ptr = ffi.NULL
 
         self._stored_exception = None
-        err = C.git_remote_fetch(self._remote, ptr, to_bytes(message))
-        if self._stored_exception:
-            raise self._stored_exception
 
-        check_error(err)
+        try:
+            err = C.git_remote_fetch(self._remote, ptr, to_bytes(message))
+            if self._stored_exception:
+                raise self._stored_exception
+
+            check_error(err)
+        finally:
+            # Even on error, clear stored callbacks and reset to default
+            self._self_handle = None
+            err = C.git_remote_set_callbacks(self._remote, defaultcallbacks)
+            check_error(err)
 
         return TransferProgress(C.git_remote_stats(self._remote))
 
@@ -291,7 +315,7 @@ class Remote(object):
     def push(self, spec, signature=None, message=None):
         """push(refspec, signature, message)
 
-        Push the given refspec to the remote. Raises ``GitError`` on error
+        Push the given refspec to the remote. Raises ``GitError`` on error.
 
         :param str spec: push refspec to use
         :param Signature signature: signature to use when updating the tips
