@@ -35,6 +35,7 @@ from string import hexdigits
 from _pygit2 import Repository as _Repository
 from _pygit2 import Oid, GIT_OID_HEXSZ, GIT_OID_MINPREFIXLEN
 from _pygit2 import GIT_CHECKOUT_SAFE_CREATE, GIT_DIFF_NORMAL
+from _pygit2 import GIT_FILEMODE_LINK
 from _pygit2 import Reference, Tree, Commit, Blob
 
 from .config import Config
@@ -42,7 +43,8 @@ from .errors import check_error
 from .ffi import ffi, C
 from .index import Index
 from .remote import Remote
-from .utils import to_bytes
+from .blame import Blame
+from .utils import to_bytes, to_str, is_string
 
 
 class Repository(_Repository):
@@ -240,11 +242,13 @@ class Repository(_Repository):
 
           >>> checkout('HEAD')
 
+        This is identical to calling checkout_head().
+
         If no reference is given, checkout from the index.
 
         Arguments:
 
-        :param str refname: The reference to checkout. After checkout,
+        :param str|Reference refname: The reference to checkout. After checkout,
           the current branch will be switched to this one.
 
         :param int strategy: A ``GIT_CHECKOUT_`` value. The default is
@@ -263,7 +267,7 @@ class Repository(_Repository):
             return self.checkout_head(**kwargs)
 
         # Case 3: Reference
-        if type(refname) is Reference:
+        if isinstance(refname, Reference):
             reference = refname
             refname = refname.name
         else:
@@ -272,7 +276,57 @@ class Repository(_Repository):
         oid = reference.resolve().target
         treeish = self[oid]
         self.checkout_tree(treeish, **kwargs)
-        self.head = refname
+        head = self.lookup_reference('HEAD')
+        if head.type == C.GIT_REF_SYMBOLIC:
+            from_ = self.head.shorthand
+        else:
+            from_ = head.target.hex
+
+        try:
+            signature = self.default_signature
+        except:
+            signature = None
+
+        reflog_text = "checkout: moving from %s to %s" % (from_, reference)
+        self.set_head(refname, signature, reflog_text)
+
+    #
+    # Setting HEAD
+    #
+    def set_head(self, target, signature=None, message=None):
+        """Set HEAD to point to the given target
+
+        Arguments:
+
+        target
+            The new target for HEAD. Can be a string or Oid (to detach)
+
+        signature
+            Signature to use for the reflog. If not provided, the repository's
+            default will be used
+
+        message
+            Message to use for the reflog
+        """
+
+        sig_ptr = ffi.new('git_signature **')
+        if signature:
+            ffi.buffer(sig_ptr)[:] = signature._pointer[:]
+
+        message_ptr = ffi.NULL
+        if message_ptr:
+            message_ptr = to_bytes(message)
+
+        if isinstance(target, Oid):
+            oid = ffi.new('git_oid *')
+            ffi.buffer(oid)[:] = target.raw[:]
+            err = C.git_repository_set_head_detached(self._repo, oid, sig_ptr[0], message_ptr)
+            check_error(err)
+            return
+
+        # if it's a string, then it's a reference name
+        err = C.git_repository_set_head(self._repo, to_bytes(target), sig_ptr[0], message_ptr)
+        check_error(err)
 
     #
     # Diff
@@ -368,6 +422,61 @@ class Repository(_Repository):
         """
         C.git_repository_state_cleanup(self._repo)
 
+    #
+    # blame
+    #
+    def blame(self, path, flags=None, min_match_characters=None, newest_commit=None, oldest_commit=None, min_line=None, max_line=None):
+        """blame(path, [flags, min_match_characters, newest_commit, oldest_commit,\n"
+                 min_line, max_line]) -> Blame
+
+        Get the blame for a single file.
+
+        Arguments:
+
+        path
+            Path to the file to blame.
+        flags
+            A GIT_BLAME_* constant.
+        min_match_characters
+            The number of alphanum chars that must be detected as moving/copying
+            within a file for it to associate those lines with the parent commit.
+        newest_commit
+            The id of the newest commit to consider.
+        oldest_commit
+          The id of the oldest commit to consider.
+        min_line
+            The first line in the file to blame.
+        max_line
+            The last line in the file to blame.
+
+        Examples::
+
+            repo.blame('foo.c', flags=GIT_BLAME_TRACK_COPIES_SAME_FILE)");
+        """
+
+        options = ffi.new('git_blame_options *')
+        C.git_blame_init_options(options, C.GIT_BLAME_OPTIONS_VERSION)
+        if min_match_characters:
+            options.min_match_characters = min_match_characters
+        if newest_commit:
+            if not isinstance(newest_commit, Oid):
+                newest_commit = Oid(hex=newest_commit)
+            ffi.buffer(ffi.addressof(options, 'newest_commit'))[:] = newest_commit.raw
+        if oldest_commit:
+            if not isinstance(oldest_commit, Oid):
+                oldest_commit = Oid(hex=oldest_commit)
+            ffi.buffer(ffi.addressof(options, 'oldest_commit'))[:] = oldest_commit.raw
+        if min_line:
+            options.min_line = min_line
+        if max_line:
+            options.max_line = max_line
+
+        cblame = ffi.new('git_blame **')
+        err = C.git_blame_file(cblame, self._repo, to_bytes(path), options)
+        check_error(err)
+
+        return Blame._from_c(self, cblame[0])
+
     #
     # Index
     #
@@ -380,3 +489,73 @@ class Repository(_Repository):
         check_error(err, True)
 
         return Index.from_c(self, cindex)
+
+    #
+    # Utility for writing a tree into an archive
+    #
+    def write_archive(self, treeish, archive, timestamp=None):
+        """Write treeish into an archive
+
+        If no timestamp is provided and 'treeish' is a commit, its committer
+        timestamp will be used. Otherwise the current time will be used.
+
+        Arguments:
+
+        treeish
+            The treeish to write.
+        archive
+            An archive from the 'tarfile' module
+        timestamp
+            Timestamp to use for the files in the archive.
+
+        Example::
+
+            >>> import tarfile, pygit2
+            >>>> with tarfile.open('foo.tar', 'w') as archive:
+            >>>>     repo = pygit2.Repsitory('.')
+            >>>>     repo.write_archive(archive, repo.head.target)
+        """
+
+        import tarfile, sys
+        from time import time
+        if sys.version_info[0] < 3:
+            from cStringIO import StringIO
+        else:
+            from io import BytesIO as StringIO
+
+        # Try to get a tree form whatever we got
+        if isinstance(treeish, Tree):
+            tree = treeish
+
+        if isinstance(treeish, Oid) or is_string(treeish):
+            treeish = self[treeish]
+
+        # if we don't have a timestamp, try to get it from a commit
+        if not timestamp:
+            try:
+                commit = treeish.peel(Commit)
+                timestamp = commit.committer.time
+            except:
+                pass
+
+        # as a last resort, use the current timestamp
+        if not timestamp:
+            timestamp = int(time())
+
+        tree = treeish.peel(Tree)
+
+        index = Index()
+        index.read_tree(tree)
+
+        for entry in index:
+            content = self[entry.id].read_raw()
+            info = tarfile.TarInfo(entry.path)
+            info.size = len(content)
+            info.mtime = timestamp
+            info.uname = info.gname = 'root' # just because git does this
+            if entry.mode == GIT_FILEMODE_LINK:
+                info.type = archive.SYMTYPE
+                info.linkname = content
+                info.mode = 0o777 # symlinks get placeholder
+
+            archive.addfile(info, StringIO(content))
