@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2010-2013 The pygit2 contributors
+# Copyright 2010-2014 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -58,7 +58,7 @@ class NotesTest(utils.BareRepoTestCase):
     def test_lookup_note(self):
         annotated_id = self.repo.head.target.hex
         note = self.repo.lookup_note(annotated_id)
-        self.assertEqual(NOTES[0][0], note.oid.hex)
+        self.assertEqual(NOTES[0][0], note.id.hex)
         self.assertEqual(NOTES[0][1], note.message)
 
     def test_remove_note(self):
@@ -70,7 +70,7 @@ class NotesTest(utils.BareRepoTestCase):
 
     def test_iterate_notes(self):
         for i, note in enumerate(self.repo.notes()):
-            entry = (note.oid.hex, note.message, note.annotated_id)
+            entry = (note.id.hex, note.message, note.annotated_id)
             self.assertEqual(NOTES[i], entry)
 
     def test_iterate_non_existing_ref(self):
