@@ -30,9 +30,15 @@
 
 import unittest
 import pygit2
+import sys
 from pygit2 import Oid
 from . import utils
 
+try:
+    import __pypy__
+except ImportError:
+    __pypy__ = None
+
 REMOTE_NAME = 'origin'
 REMOTE_URL = 'git://github.com/libgit2/pygit2.git'
 REMOTE_FETCHSPEC_SRC = 'refs/heads/*'
@@ -56,6 +62,17 @@ class RepositoryTest(utils.RepoTestCase):
 
         self.assertRaises(ValueError, self.repo.create_remote, *(name, url))
 
+    def test_remote_delete(self):
+        name = 'upstream'
+        url = 'git://github.com/libgit2/pygit2.git'
+
+        self.repo.create_remote(name, url)
+        self.assertEqual(2, len(self.repo.remotes))
+        remote = self.repo.remotes[1]
+
+        self.assertEqual(name, remote.name)
+        remote.delete()
+        self.assertEqual(1, len(self.repo.remotes))
 
     def test_remote_rename(self):
         remote = self.repo.remotes[0]
@@ -163,6 +180,14 @@ class RepositoryTest(utils.RepoTestCase):
         self.assertEqual('http://example.com/test.git',
                          self.repo.remotes[0].url)
 
+    @unittest.skipIf(__pypy__ is not None, "skip refcounts checks in pypy")
+    def test_remote_refcount(self):
+        start = sys.getrefcount(self.repo)
+        remote = self.repo.remotes[0]
+        del remote
+        end = sys.getrefcount(self.repo)
+        self.assertEqual(start, end)
+
     def test_add_refspec(self):
         remote = self.repo.create_remote('test_add_refspec', REMOTE_URL)
         remote.add_push('refs/heads/*:refs/heads/test_refspec/*')
