@@ -37,6 +37,7 @@ import unittest
 import tempfile
 import os
 from os.path import join, realpath
+import sys
 
 # Import from pygit2
 from pygit2 import GIT_OBJ_ANY, GIT_OBJ_BLOB, GIT_OBJ_COMMIT
@@ -45,6 +46,11 @@ from pygit2 import Oid, Reference, hashfile
 import pygit2
 from . import utils
 
+try:
+    import __pypy__
+except ImportError:
+    __pypy__ = None
+
 
 HEAD_SHA = '784855caf26449a1914d2cf62d12b9374d76ae78'
 PARENT_SHA = 'f5e5aa4e36ab0fe62ee1ccc6eb8f79b866863b87'  # HEAD^
@@ -70,10 +76,10 @@ class RepositoryTest(utils.BareRepoTestCase):
 
     def test_set_head(self):
         # Test setting a detatched HEAD.
-        self.repo.head = Oid(hex=PARENT_SHA)
+        self.repo.set_head(Oid(hex=PARENT_SHA))
         self.assertEqual(self.repo.head.target.hex, PARENT_SHA)
         # And test setting a normal HEAD.
-        self.repo.head = "refs/heads/master"
+        self.repo.set_head("refs/heads/master")
         self.assertEqual(self.repo.head.name, "refs/heads/master")
         self.assertEqual(self.repo.head.target.hex, HEAD_SHA)
 
@@ -150,6 +156,15 @@ class RepositoryTest(utils.BareRepoTestCase):
             commit.message)
         self.assertRaises(ValueError, self.repo.__getitem__, too_short_prefix)
 
+    @unittest.skipIf(__pypy__ is not None, "skip refcounts checks in pypy")
+    def test_lookup_commit_refcount(self):
+        start = sys.getrefcount(self.repo)
+        commit_sha = '5fe808e8953c12735680c257f56600cb0de44b10'
+        commit = self.repo[commit_sha]
+        del commit
+        end = sys.getrefcount(self.repo)
+        self.assertEqual(start, end)
+
     def test_get_path(self):
         directory = realpath(self.repo.path)
         expected = realpath(self.repo_path)
@@ -221,6 +236,25 @@ class RepositoryTest_II(utils.RepoTestCase):
         self.assertTrue('new' in head.tree)
         self.assertTrue('bye.txt' not in self.repo.status())
 
+    def test_checkout_branch(self):
+        branch_i18n = self.repo.lookup_branch('i18n')
+
+        # checkout i18n with conflicts and default strategy should
+        # not be possible
+        self.assertRaises(pygit2.GitError, self.repo.checkout, branch_i18n)
+
+        # checkout i18n with GIT_CHECKOUT_FORCE
+        head = self.repo.head
+        head = self.repo[head.target]
+        self.assertTrue('new' not in head.tree)
+        self.repo.checkout(branch_i18n, strategy=pygit2.GIT_CHECKOUT_FORCE)
+
+        head = self.repo.head
+        head = self.repo[head.target]
+        self.assertEqual(head.hex, branch_i18n.target.hex)
+        self.assertTrue('new' in head.tree)
+        self.assertTrue('bye.txt' not in self.repo.status())
+
     def test_checkout_index(self):
         # some changes to working dir
         with open(os.path.join(self.repo.workdir, 'hello.txt'), 'w') as f:
@@ -428,6 +462,17 @@ class CloneRepositoryTest(utils.NoRepoTestCase):
 
         self.assertFalse(repo.is_empty)
 
+    def test_clone_with_checkout_branch(self):
+        # create a test case which isolates the remote
+        test_repo = clone_repository('./test/data/testrepo.git',
+                                     os.path.join(self._temp_dir, 'testrepo-orig.git'),
+                                     bare=True)
+        test_repo.create_branch('test', test_repo[test_repo.head.target])
+        repo = clone_repository(test_repo.path,
+                                os.path.join(self._temp_dir, 'testrepo.git'),
+                                checkout_branch='test', bare=True)
+        self.assertEqual(repo.lookup_reference('HEAD').target, 'refs/heads/test')
+
     # FIXME The tests below are commented because they are broken:
     #
     # - test_clone_push_url: Passes, but does nothing useful.
@@ -437,8 +482,6 @@ class CloneRepositoryTest(utils.NoRepoTestCase):
     #
     # - test_clone_push_spec: Passes, but does nothing useful.
     #
-    # - test_clone_checkout_branch: Fails, because the test fixture does not
-    #   have any branch named "test"
 
 #   def test_clone_push_url(self):
 #       repo_path = "./test/data/testrepo.git/"
@@ -470,15 +513,5 @@ class CloneRepositoryTest(utils.NoRepoTestCase):
 #       # not sure how to test this either... couldn't find pushspec
 #       # self.assertEqual(repo.remotes[0].fetchspec, "refs/heads/test")
 
-#   def test_clone_checkout_branch(self):
-#       repo_path = "./test/data/testrepo.git/"
-#       repo = clone_repository(repo_path, self._temp_dir,
-#                               checkout_branch="test")
-#       self.assertFalse(repo.is_empty)
-#       # FIXME: When pygit2 supports retrieving the current branch,
-#       # enable this test
-#       # self.assertEqual(repo.remotes[0].current_branch, "test")
-
-
 if __name__ == '__main__':
     unittest.main()
